;################################################################# LIBTEST.MAC
; Purpose.....: Test the library functions. Program can be terminated with
;		SHIFT and CONTROL keys pressed simultaneously, except for
;		delay calibration, which requires a RESET.
; Target......: MPF-1P
; Assembler...: ASL V1.42 Beta
; Command Line: ASL -listline-prefix %04a -L -U -olist <name>.prn <name>.asl
; PsPadParser.: > > > %F(%L):%C: error:
;#############################################################################

;********************************************************************* history
;=========================================================== V01.00:25.03.2024
; Created
;=========================================================== V01.10:03.04.2024 
; -Warm boot signature at start of RAM
; -Sign-on message
;============================================================= V01.20:08.04.24
; -Check if printer connected before printing in line editor
; -Moved variables to MPF RAM to get continuous RAM from D800 to FD00
;=========================================================== V01.20:27.08.2024
; Changed to ASL assembler, no code change
;=========================================================== V01.30:30.08.2024
; Removed the program from ROM, make it runnable at xx00 addresses
;***************************************************************** end history

;*********************************************************************** notes
;=========================================================== library debugging
; To be able to debug the library code, its code must be in RAM. Set the
; equate dbLTAR in MPF1P.DEF to dbLDEB, then assemble and upload the
; library code to E000 (or whatever is definded in debug's dwLCS).
;=================================================================== operating
; PA0 is toggled on 10mS CTC timer 0 interrupt, in callback
; PA1            on 20mS           1            in callback
; PA2            on 30mS           3            in callback
; PA3            on      PIO-PB7 interrupt, in callback
; PB0            on 10mS CTC timer 0 interrupt /2,  in main loop
; PB1            on 20mS           1           /16, in main loop
; PB2            on 30mS           2           /16, in main loop
; PB3            on      PIO-PB7 interrupt     /3,  in main loop
;
; PB7 is setup as interrupt input on rising edge. Connect PAx to PB7 to 
; trigger PIO-B interrupts
;
; USART is setup for 1200,N,8,1 with RTS/CTS handshake
;
; DIP-SW1 on : lUDLY() calibration. Toggles PIO B0 pin in 10mS intervals
;         off: Calibration is off
; DIP-SW2 on : Interrupt counters are displayed if SW1 is off. CTC-0 and CTC-1
;              as HEX, CTC-3 as decimal with leading zeroes, PIO-B as decimal
;              with leading spaces.
;         off: No display of interrupt counters.
; DIP-SW3 on : Line editor. if SW1 and SW2 are off. Ctrl-P prints the edited
;              line
;         off: Line editor is off
; DIP-SW4 on : USART echoes received characters back to sender. Blocks 
;              mainloop on continuos receive (no interrupt operation possible)
;         off: USART is silent
; DIP-SW8 on : Interrupts enabled  (SW8 is change-triggered)
;         off: Interrupts disabled
;******************************************************************* end notes

;***************************************************************** definitions
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++ assembler directives
		CPU	Z80		; use Z80 CPU
		PAGE	0		; no formfeeds in listing file
		
;+++++++++++++++++++++++++++++++++++ hardware, monitor and library definitions
		LISTING	OFF
		INCLUDE	../LIBRARY/MPF1P.DEF
		LISTING ON
		
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ build addresses
; The code contains jumps over IO addresses. It must be assembled at code
; addresses xx00, otherwise it will crash due to improper I/O decoding.
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
wLTCS:		EQU	0F000H		; code: xx00 
wLTDS:		EQU	0FD00H		; where data goes
wLTWB:		EQU	0AA55H		; warmboot signature

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ variables
vLTWB:		EQU	wLTDS		; warm boot signature
vLTHLT:		EQU	vLTWB  + 2	; address of current help line
vLTHLC:		EQU	vLTHLT + 2	; help line counter
vLTDC:		EQU	vLTHLC + 1	; delay calib loop value

vLTDI0:		EQU	vLTDC  + 2	; CTC-0 ints counter
vLTDI1:		EQU	vLTDI0 + 2	; CTC-1 ints counter
vLTDI3:		EQU	vLTDI1 + 2	; CTC-3 ints counter
vLTDIP:		EQU	vLTDI3 + 2	; PIO-B ints counter

vLTF:		EQU	vLTDIP + 2	; flags:
iLTHLP:		EQU	0		;  bit0: help display running
iLTEDC:		EQU	1		;  bit1: editor in continue mode
iLTEDD:		EQU	2		;  bit2: editor in decimal mode
iLTEDH:		EQU	3		;  bit3: editor in HEX mode
iLTUER:		EQU	4		;  bit4: USART echo running
vLTOSW:		EQU	vLTF   + 1	; DIP switch status
iLTODC:		EQU	0		;  SW1: lUDLY() calibration
iLTOIC:		EQU	1		;  SW2: ints counter
iLTOET:		EQU	2		;  SW3: line editor
iLTOUS:		EQU	3		;  SW4: USART echo on/off
iLTOIN:		EQU	7		;  SW8: ints on/off
vLTUBR:		EQU	vLTOSW  + 1	; USART baudrate

wLTVIS:		EQU	vLTHLT		; init start address
wLTVIE:		EQU	vLTF		; init end address
wLTVIZ:		EQU	wLTVIE-wLTVIS	; variables init size
wLTVE:		EQU	vLTUBR		; variables end address

;==================================== check for overlap with library variables
		IF	wLTVE > dwLVST
		WARNING	"Variables overlap with library:\{wLTVE} >> \{dwLVST}"
		ENDIF
;************************************************************* end definitions

;************************************************************** implementation
		ASEG
		ORG	wLTCS
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ main program
;======================================================================== LT()
; Purpose.....: Main program
; Labels......: L)ibrary T)est
;=============================================================================
LT:		CALL	LTIC		; cold boot initialization
		JR	LTM		; goto main loop

LTW:		CALL	LTIW		; warm boot entry point
		JR	C,LT		; signature not found, do cold boot

;---------------------------------------- entry after cold/warm initialization
LTM:		LD	IY,vLTF		; IY= flags address
		XOR	A
		LD	(vLTF),A	; reset all flags

		LD	A,2		; use IM2
		CALL	dfLIS		; place CPU in interrupt mode

;------------------------------------------------------------ setup CTC timers
		LD	D,dbIC0		; timer 0
		LD	E,dbCI2		; timer mode, interrupt, prescaler 256
		LD	B,dbCO10	; 10mS interval
		LD	HL,LTC0CB	; timer 0 callback address
		SCF			; start timer
		CALL	dfLCS		; setup timer 0

		LD	D,dbIC1		; timer 1
		LD	E,dbCI2		; timer mode, interrupt, prescaler 256
		LD	B,dbCO20	; 20mS interval
		LD	HL,LTC1CB	; timer 1 callback address
		SCF			; start timer
		CALL	dfLCS		; setup timer 1

		LD	D,dbIC3		; timer 3
		LD	E,dbCI2		; timer mode, interrupt, prescaler 256
		LD	B,dbCO30	; 30mS interval
		LD	HL,LTC3CB	; timer 3 callback address
		SCF			; start timer
		CALL	dfLCS		; setup timer 3

;------------------------------------------------------------------- setup PIO
		LD	D,dbIPA		; channel A
		LD	E,dbPMO		; mode 0, output
		LD	B,dbPID		; disable interrupts
		SCF			; set it up
		CALL	dfLPS		; setup PIO channel A

		LD	D,dbIPB		; channel B
		LD	E,dbPMC		; mode 3, control
		LD	B,dbPIOH	; interrupts, OR mask, hi state
		LD	C,01111111B	; int on PB7 hi transition
		LD	A,11110000B	; pins 7-4 inputs, 3-0 outputs
		LD	HL,LTPBCB	; set callback address
		SCF			; set it up
		CALL	dfLPS		; setup PIO channel B

;------------------------------------------------------ read DIP switch on IOM
LTS:		CALL	dfLOSW		; int switch is event driven,
		LD	C,A		; ints will not start on startup
		JR	LTSWI		; if switch is already on

LTL:		JR	ISIOM1		;-------------------------------------
		DS	0FH		; $$$ IOM-MPF: skip I/O xx6x
ISIOM1:		EQU	$		;-------------------------------------

		LD	A,10000000B	; change notification for ints
		CALL	dfLOSW		; get switches
		LD	C,A		; store status
		JR	Z,LTSC		; int enable switch has not changed

;--------------------------------------------- interrupts switch changed state
LTSWI:		DI			; int off in any case
		BIT	iLTOIN,C	; ints requested?
		JR	Z,LTSC		; no
		EI			; yes, enable them

;------------------------------------------------- read SHIFT and CONTROL keys
LTSC:  		CALL	dfLKSOC		; get shift or control key status
		JR	NC,LTSW		; control not pressed, continue
		JR	NZ,LTSW		; shift not pressed, continue
		JR	LTX		; shift and control pressed, exit

;----------------------- display help message if all function switches are off
LTSW:		LD	A,C		; A= DIP switch status
		AND	10001111B	; any function active?
		JR	NZ,LTPIC	; yes, continue
		CALL	LTHLP		; no, call help

;----------------------------------------- show interrupt counters on PIO B0-3
LTPIC:		CALL	LTPI

;------------------------------------------------ calibrate lHDLY() delay time
LTSWD:		BIT	iLTODC,C
		JR	Z,LTSWC		; skip if SW1 is off

		CALL	LTDC		; start calibration after '>' key

;-------------------------------------------------- display interrupt counters
LTSWC:		LD	A,C		; A= DIP switch
		AND	00000011B	; mask allowed bits
		CP	00000010B	; may not run if calibrating runs
		JR	NZ,LTSWE

		CALL	LTDI		; run counter display

;--------------------------------------------------- line editor with printing
LTSWE:		LD	A,C		; A= DIP switch
		AND	00000111B	; mask allowed bits
		CP	00000100B	; may not run if calib or cnt run
		JR	NZ,LTSWER

		CALL	LTED		; run text editor
		JR	LTSWU		; skip reset continue flag

LTSWER:		RES	iLTEDC,(IY)	; reset continued calls flag

;--------------------------------------- read and echo character from/to USART
LTSWU:		BIT	iLTOUS,C
		JR	Z,LTSWUF	; handshake off if USART is inactive

LTSWUG:		CALL	LTUS		; call USART handler
		JR	LTL		; and back to loop

LTSWUF:  	BIT	iLTUER,(IY)	; already inactive?
		JR	Z,LTL		; yes, back to loop
		XOR	A		; no, set inactive
		CALL	dfLURTS		; deactivate RTS

  		JR	ISPRT1		;-------------------------------------
  		DS	04H		; $$$ PRT-MPF: skip I/O xxCA, CB
ISPRT1:		EQU	$		;-------------------------------------

		RES	iLTUER,(IY)	; reset message flag
		JR	LTL		; and back to loop

;------------------------------------------------------------------- terminate
LTX:		DI			; disable interrupts

		XOR	A
		CALL	dfLURTS		; disable RTS

		XOR	A
		CALL	dfLUDTR		; disable DTR

		XOR	A
		LD	D,dbIC0
		CALL	dfLCS		; stop timer 0

		XOR	A
		LD	D,dbIC1
		CALL	dfLCS		; stop timer 1

		XOR	A
		LD	D,dbIC3
		CALL	dfLCS		; stop timer 3

		XOR	A
		LD	D,dbIPA
		CALL	dfLPS		; reset PIO-A

		XOR	A
		LD	D,dbIPB
		CALL	dfLPS		; reset PIO-B

		JP	dwMRET		; monitor warm start
;==================================================================== eoM LT()
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end main program

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ subroutines
;====================================================================== LTIC()
; Purpose.....: Cold boot initialization
; Registers...: None saved
; Labels......: L)ibrary T)est I)nit C)old
;=============================================================================
LTIC:		CALL	dfLLS		; setup library variables

		LD	HL,wLTVIS	; init start address
		LD	BC,wLTVIZ	; init size
		XOR	A		; init value
		CALL	dfLHMF		; fill memory

		LD	HL,wLTWB	; setup warmboot signature
		LD	(vLTWB),HL

		LD	A,20		; A= value for delay calib (10mS)
		LD	(vLTDC),A

		LD	A,dbUB1K	; A= default USART baudrate
		LD	(vLTUBR),A

		LD	IX,tLTSO	; sign-on message
		LD	B,040H		; 30H ~1s
		CALL	dfLDT		; show it

		RET
;================================================================== eoF LTIC()

;====================================================================== LTIW()
; Purpose.....: Warm boot initialization, check if signature present
; Output......: CY: 1= warm boot signature not found
; Registers...: None saved
; Labels......: L)ib T)est I)nit W)arm
;=============================================================================
LTIW:		CALL	dfLLS		; setup library variables
		LD	DE,wLTWB	; warmboot signature
		LD	HL,(vLTWB)	; HL= stored signature
		XOR	A		; clear CY
		SBC	HL,DE
		JR	Z,LTIWX		; signature valid, exit with CY=0
		SCF			; no match, return with CY=1

LTIWX:		RET
;================================================================== eoF LTIW()

;===================================================================== LTHLP()
; Purpose.....: Display help messages if all switches are off
; Output......: CY: 1= ctrl-Q was pressed
; Registers...: None saved
; Labels......: L)ib T)est H)el)p)
;=============================================================================
LTHLP:		BIT	iLTHLP,(IY)	; help already running?
		JR	NZ,LTHLPL	; yes
		SET	iLTHLP,(IY)	; no, set help-running flag

;----------------------------------------------- construct LIB version message
		LD	HL,wLTVERPM	; HL= position of major version
		LD	A,(dbLVMAJ)	; A= major version
		LD	E,A		; value lobyte
		LD	D,0		; value hibyte
		LD	C,2		; two digits
		SCF			; leading zeros
		CALL	dfLOBD		; convert to decimal
		LD	HL,wLTVERPI	; HL= position of minor version
		LD	A,(dbLVMIN)	; A= minor version
		LD	E,A		; value lobyte
		SCF			; leading zeros
		CALL	dfLOBD		; convert to decimal

		LD	HL,tLTVER	; show 1st message
		LD	(vLTHLT),HL

LTHLPL:		LD	IX,(vLTHLT)	; IX= address of current message
		CALL	dfLDK		; display it
		CP	dbAR		; right arrow key?
		JR	NZ,LTHLPX	; no, exit

 		LD	HL,(vLTHLT)	; HL= address of current message
		LD	A,(vLTHLC)	; A= messages counter
		INC	A		; bump it
		CP	3		; max reached?
		JR	NZ,LTHLPC	; no, point to next

		LD	HL,tLTVER-dbMCPL; yes, point to first
		XOR	A		; and reset counter
LTHLPC:		LD	(vLTHLC),A	; write it back

LTHLPB:		LD	DE,dbMCPL	; bump message pointer
		ADD	HL,DE
		LD	(vLTHLT),HL	; HL points to next message

LTHLPX:		RET
;================================================================= eoF LTHLP()

;====================================================================== LTDC()
; Purpose.....: Calibrate lHDLY() delay. A 10mS (default) pulse is output
;		on PB0. Can be changed wy writing another value to (vLTDC).
;		Start with right-arrow key.
; Registers...: AF destroyed
; Labels......: L)ib T)est D)elay C)alibrate
;=============================================================================
LTDC:		LD	IX,tLTDC	; prompt message
		CALL	dfLDK		; display it
		JR	Z,LTDCX		; no key pressed, exit

		CP	dbAR		; right arrow?
		JR	Z,LTDCG		; yes, start calibration, no return
		JR	LTDCX		; no, exit

LTDCG:		NOP			; placeholder for DI

LTDCL:		LD	DE,(vLTDC)	; 20*500uS = 10mS
		IN	A,(dpIPBD)	; A= PIO-A data
		XOR	01H		; invert bit PA7
		CALL	dfLHDLY		; delay 1mS
		OUT	(dpIPBD),A	; write back to PIO-A
		JR	LTDCL

LTDCX:		RET
;================================================================== eoF LTDC()

;====================================================================== LTPI()
; Purpose.....: Visualize interrupts on PIO port B by reading the occurence
;		counters provided by the library interrupt service routines.
;		PIO PB0 is toggled by timer 0 interrupt / 2
;		      1			    1		/ 16
;		      2			    3		/ 16
;		      3		      PIO B7 interrupt  / 3
; Registers...:	All saved
; Labels......: L)ib T)est P)IO I)nterrupts
;=============================================================================
LTPI:		PUSH	BC
		PUSH	AF

		LD	B,0		; bit toggle mask

LTPI0:		LD	A,(dvLCIC0)	; timer 0 ints counter
		CP	01H
		JR	C,LTPI1		; not reached, check timer 1
		XOR	A		; reached, reset counter
		LD	(dvLCIC0),A
		SET	0,B		; toggle PB0

LTPI1:		LD	A,(dvLCIC1)	; timer 1 ints counter
		CP	010H
		JR	C,LTPI3		; not reached, check timer 3
		XOR	A		; reached, reset counter
		LD	(dvLCIC1),A
		SET	1,B		; toggle PB1

LTPI3:		LD	A,(dvLCIC3)	; timer 3 ints counter
		CP	010H
		JR	C,LTPIP		; not reached, check PIO B
		XOR	A		; reached, reset counter

		JR	ISPRT2		;-------------------------------------
		DS	02H		; $$$ PRT-MPF: skip I/O xxCA, CB
ISPRT2:		EQU	$		;-------------------------------------
		
		LD	(dvLCIC3),A
		SET	2,B		; toggle PB2

LTPIP:		LD	A,(dvLPICB)	; PIO channel B ints counter
		CP	02H
		JR	C,LTPIO		; not reached, write to PIO B
		XOR	A		; reached, reset counter
		LD	(dvLPICB),A
		SET	3,B		; toggle PB3

LTPIO:		IN	A,(dpIPBD)	; read channel B data port
		XOR	B		; invert requested bits
		OUT	(dpIPBD),A	; and write it back

		POP	AF
		POP	BC
		RET
;================================================================== eoF LTPI()

;====================================================================== LTDI()
; Purpose.....: Display interrupt counters provided by the interrupt
;		callback routines, in HEX and decimal
; Registers...:	AF, IX destroyed
; Labels......: L)ib T)est D)isplay I)interrupts
;=============================================================================
LTDI:		PUSH	HL
		PUSH	DE
		PUSH	BC

		LD	C,4		; digits to convert

		LD	HL,tLTDI	; HL= text buffer address
		LD	DE,(vLTDI0)	; DE= CTC-0 counter
		CALL	dfLOBH		; convert to HEX

		LD	HL,tLTDI+5
		LD	DE,(vLTDI1)	; DE= CTC-1 counter
		CALL	dfLOBH		; convert to HEX

		LD	HL,tLTDI+10
		LD	DE,(vLTDI3)	; DE= CTC-3 counter
		SCF			; leading zeros
		CALL	dfLOBD		; convert to decimal

		LD	HL,tLTDI+15
		LD	DE,(vLTDIP)	; DE= PIO-B counter
		OR	A		; reset CY, leading blanks
		CALL	dfLOBD		; convert to decimal

		LD	IX,tLTDI	; IX= start of display buffer
		CALL	dfLDK		; display and scan keyboard

LTDIX:		POP	BC
		POP	DE
		POP	HL
		RET
;================================================================== eoF LTDI()

;====================================================================== LTED()
; Purpose.....: Call the line editor in text, decimal and hex modes
; Output......: A: last pressed key, 0 if none
; Registers...:	AF,IX destroyed
; Labels......: L)ib T)est E)d)itor
;=============================================================================
LTED:		PUSH	BC

		BIT	iLTEDC,(IY)	; continue mode flag
		LD	IX,0		; preset to continued
		JR	NZ,LTEDM	; it's continued
		LD	IX,tLTED	; it's initial, get buffer address

LTEDM:		BIT	iLTEDD,(IY)	; decimal mode active?
		JR	NZ,LTEDD	; yes
		BIT	iLTEDH,(IY)	; hex mode active?
		JR	NZ,LTEDH	; yes

;------------------------------------------------------------------- text mode
LTEDT:		XOR	A		; no buffer clear/text mode
		LD	B,9		; nine characters
		LD	C,bLTEDT	; cursor position
		CALL	dfLDKR		; call library editor
		SET	iLTEDC,(IY)	; set continue flag
		JR	Z,LTEDTC	; CR pressed, switch mode
		CP	dbCTP		; ctrl-P pressed?
		JR	NZ,LTEDX	; no, exit
		JR	LTEDP		; yes, print the line

LTEDTC:		SET	iLTEDD,(IY)	; CR pressed, switch to decimal mode
		JR	LTEDXC		; clear continue flag

;---------------------------------------------------------------- decimal mode
LTEDD:		XOR	A		; no buffer clear
		LD	A,1		; set decimal mode
		LD	B,5		; use five digits
		LD	C,bLTEDD	; cursor position
		CALL	dfLDKR		; call editor

  		JR	ISIOM2		;-------------------------------------
  		DS	0FH		; $$$ IOM-MPF: skip I/O xx6x
ISIOM2:		EQU	$		;-------------------------------------

		SET	iLTEDC,(IY)	; set continue flag ($$$ in operands)
		JR	NZ,LTEDX	; CR not pressed, exit
		RES	iLTEDD,(IY)	; clear decimal mode
		SET	iLTEDH,(IY)	; CR pressed, switch to HEX mode
		JR	LTEDXC		; clear continue flag

;-------------------------------------------------------------------- hex mode
LTEDH:		XOR	A		; no buffer clear
		LD	A,2		; set hex mode
		LD	B,4		; use four digits
		LD	C,bLTEDH	; cursor position
		CALL	dfLDKR		; call editor
		SET	iLTEDC,(IY)	; set continue flag
		JR	NZ,LTEDX	; CR not pressed, exit
		RES	iLTEDH,(IY)	; CR pressed, switch to text mode
		JR	LTEDXC		; clear continue flag

;-------------------------------------------------------------- print the line
LTEDP:		CALL	dfPCN		; PRT-MPF connected?
		JR	NZ,LTEDXC	; no, exit

		CALL	dfLID		; ints off
		RL	C		; save ints-on flag
		LD	IX,tLTED	; IX= text buffer address
		CALL	dfPPLB		; print the edited line
		RR	C		; recall ints enabled flag
		JR	NC,LTEDXC	; were not enabled, leave them off
		EI			; were enabled, back on

LTEDXC:		RES	iLTEDC,(IY)	; CR or print, clear continue flag

LTEDX:		POP	BC
		RET
;================================================================== eoF LTED()

;====================================================================== LTUS()
; Purpose.....: Read USART and echo back if a char was received
; Registers...:	AF destroyed
; Labels......: L)ib T)est U)S)ART
;=============================================================================
LTUS:		PUSH	DE

		BIT	iLTUER,(IY)	; initial call?
		JR	NZ,LTUSRL	; no, continue
		SET	iLTUER,(IY)	; yes, set to not initial

		LD	A,(vLTUBR)	; get baudrate
		LD	B,A
		LD	C,dbU8N1	; 8 data, no parity, 1 stop
		CALL	dfLID		; disable interrupts

		PUSH	AF		; save int enabled state
		CALL	dfLUS		; setup USART with ints disabled
		POP	AF

		JR	NC,LTUSM	; ints were not enabled, skip EI
		EI			; ints were enabled

LTUSM:		PUSH	IX
		LD	IX,tLTUE	; show message
		LD	B,040H		; for a second
		CALL	dfLDT
		POP	IX

		SCF			; turn RTS on
		CALL	dfLURTS

LTUSRL:		CALL	dfLURB		; get byte from USART
		JR	Z,LTUSX		; Z=1: no character ready, exit

LTUSTL:		CALL	dfLUTB		; echo it back
		JR	C,LTUSTL	; wait until sent

		LD	D,040H		; value depends on baudrate and host
LTUSDY:		LD	B,080H
		DJNZ	$		; give time to receive next char

		CALL	dfLOSW		; check for USART switch active
		BIT	iLTOUS,A	; is not checked if locked by
		JR	Z,LTUSX		; continuous receives

		CALL	dfLURB		; try to get next
		JR	NZ,LTUSTL	; ready, echo
		DEC	D		; dec outer delay counter
		JR	NZ,LTUSDY

LTUSX:		POP	DE
		RET
;================================================================== eoF LTUS()
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end subroutines

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++ interrupt callbacks
;==================================================================== LTC0CB()
; Purpose.....: CTC timer 0 interrupt callback. Runs in interrupt context
; Registers...: All saved
; Labels......: L)ib T)est C)TC timer 0) C)allB)ack
;=============================================================================
LTC0CB:		PUSH	HL
		PUSH	AF

		IN	A,(dpIPAD)	; read channel A data port
		XOR	01H		; invert bit 0
		OUT	(dpIPAD),A	; and write it back

		LD	HL,(vLTDI0)	; pump ints counter
		INC	HL
		LD	(vLTDI0),HL

LTC0CX:		POP	AF
		POP	HL
		RET
;================================================================ eoF LTC0CB()

;==================================================================== LTC1CB()
; Purpose.....: CTC timer 1 interrupt callback. Runs in interrupt context
; Registers...: All saved
; Labels......: L)ib T)est C)TC timer 1) C)allB)ack
;=============================================================================
LTC1CB:		PUSH	HL
		PUSH	AF

		IN	A,(dpIPAD)	; read channel A data port
		XOR	02H		; invert bit 1
		OUT	(dpIPAD),A	; and write it back

		LD	HL,(vLTDI1)	; pump ints counter
		INC	HL
		LD	(vLTDI1),HL

LTC1CX:		POP	AF
		POP	HL
		RET
;================================================================ eoF LTC1CB()

;==================================================================== LTC3CB()
; Purpose.....: CTC timer 3 interrupt callback. Runs in interrupt context
; Registers...: All saved
; Labels......: L)ib T)est C)TC timer 3) C)allB)ack
;=============================================================================
LTC3CB:		PUSH	HL
		PUSH	AF

		IN	A,(dpIPAD)	; read channel A data port
		XOR	04H		; invert bit 2
		OUT	(dpIPAD),A	; and write it back

		LD	HL,(vLTDI3)	; pump ints counter
		INC	HL
		LD	(vLTDI3),HL

LTC3CX:		POP	AF
		POP	HL
		RET
;================================================================ eoF LTC3CB()

;==================================================================== LTPBCB()
; Purpose.....: PIO channel B interrupt callback. Runs in interrupt context
; Registers...: All saved
; Labels......: L)ib T)est P)IO channel B) C)allB)ack
;=============================================================================
LTPBCB:		PUSH	HL
		PUSH	AF

		IN	A,(dpIPAD)	; read channel A data port
		XOR	08H		; invert bit 3
		OUT	(dpIPAD),A	; and write it back

		LD	HL,(vLTDIP)	; pump ints counter
		INC	HL
		LD	(vLTDIP),HL

LTPBCX:		POP	AF
		POP	HL
		RET
;================================================================ eoF LTPBCB()
;+++++++++++++++++++++++++++++++++++++++++++++++++++++ end interrupt callbacks

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ text constants
tLTSO:		DB	'   LIBT V24-08-27   '
tLTVER:		DB	'     LIB V--.--    )'
wLTVERPM:	EQU	tLTVER + 10
wLTVERPI:	EQU	tLTVER + 13
tLTHL1:		DB	' 1-DLY 2-CNT 3-EDT )'
tLTHL2:		DB	' 4-SER       8-INT )'
tLTDC:		DB	'10MS-PB0  EXIT-RST )'
tLTUE:		DB	'     USART ECHO     '
tLTDI:		DB	'---- ---- ---- ---- '

;---------------------------- texts which can be printed (must have CR at end)
tLTED:		DB	'TEXT      12345 ABCD', dbCR
bLTEDT:		EQU	0	       ; input position text
bLTEDD:		EQU	10	      ; input position decimal
bLTEDH:		EQU	16	      ; input position hex
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end text constants

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ print size info
wLTCSE:		EQU	$
		IF	MomPass = 2
		MESSAGE "Code from \{wLTCS} to \{wLTCSE} Size \{wLTCSE-wLTCS}"
		MESSAGE "Data from \{wLTDS} to \{wLTVE} Size \{wLTVE-wLTDS}"
		MESSAGE	"     remaining \{dwLVST-wLTVE}"
		ENDIF

;############################################################# EOF LIBTEST.ASL
		END
